[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet RBICS client library for Python

[![API Version](https://img.shields.io/badge/api-v1.1.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetRBICS/1.0.3)](https://pypi.org/project/fds.sdk.FactSetRBICS/v/1.0.3)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

FactSet Revere Business Industry Classification System (RBICS) is a comprehensive structured taxonomy to classify companies by what they primarily do. It delivers a granular view for investors by classifying companies using a bottom-up approach according to the products and services they provide. By combining this approach with a top-level grouping based on companies' behavior similarities and stock co-movement, FactSet RBICS delivers unprecedented precision for 1,400+ sector groups.

RBICS Focus is a six level classification which highlights the companys main business. It is based on the company business line/s that stand for majority of a the revenue that the company generates. Updated monthly. For more details, visit [Online Assitant Page #20710](https://oa.apps.factset.com/pages/20710).<p><b>Rate limit is set to 40 requests per second</b>.</p>


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.1.0
- SDK version: 1.0.3
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetRBICS==1.0.3
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetRBICS==1.0.3
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetRBICS
from fds.sdk.FactSetRBICS.api import entity_focus_api
from fds.sdk.FactSetRBICS.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetRBICS.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetRBICS.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetRBICS.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = entity_focus_api.EntityFocusApi(api_client)
    ids = ["AAPL-US","0FPWZZ-E","TSLA-US"] # [str] | List of Company identifiers. Accepted identifiers include Ticker-Exchange, Ticker-Regions, CUSIPs, ISINs, SEDOLs, or FactSet Permanent Ids, such as -R, -L, or -E.<p>**ids limit = 2500 per request**</p>    Make note, GET Method URL request lines are also limited to a total length of 8192 bytes (8KB). In cases where the service allows for thousands of ids, which may lead to exceeding this request line limit of 8KB, its advised for any requests with large request lines to be requested through the respective \"POST\" method.</p> 
    date = "2017-07-24" # str | Effective date for data expressed in YYYY-MM-DD format. If no date is requested, the default behavior is to return the full history for the requested entity. (optional)
    levels = [1,3,6] # [int] | List of RBICS industry levels to include in the response. **By default if left blank, all levels are returned.** (optional)
    include_names = True # bool | Option to include or exclude industry Names and the L6 Description. true = Include Names; false = Exclude Names. (optional) if omitted the server will use the default value of True

    try:
        # Get RBICS classification for the Focus industry
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_rbics_entity_focus(ids, date=date, levels=levels, include_names=include_names)

        pprint(api_response)
    except fds.sdk.FactSetRBICS.ApiException as e:
        print("Exception when calling EntityFocusApi->get_rbics_entity_focus: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Get RBICS classification for the Focus industry
    #     api_response, http_status_code, response_headers = api_instance.get_rbics_entity_focus_with_http_info(ids, date=date, levels=levels, include_names=include_names)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetRBICS.ApiException as e:
    #     print("Exception when calling EntityFocusApi->get_rbics_entity_focus: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Get RBICS classification for the Focus industry
    #     async_result = api_instance.get_rbics_entity_focus_async(ids, date=date, levels=levels, include_names=include_names)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetRBICS.ApiException as e:
    #     print("Exception when calling EntityFocusApi->get_rbics_entity_focus: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Get RBICS classification for the Focus industry
    #     async_result = api_instance.get_rbics_entity_focus_with_http_info_async(ids, date=date, levels=levels, include_names=include_names)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetRBICS.ApiException as e:
    #     print("Exception when calling EntityFocusApi->get_rbics_entity_focus: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetRBICS

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetRBICS.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetRBICS

configuration = fds.sdk.FactSetRBICS.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetRBICS

configuration = fds.sdk.FactSetRBICS.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetRBICS

configuration = fds.sdk.FactSetRBICS.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*EntityFocusApi* | [**get_rbics_entity_focus**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/EntityFocusApi.md#get_rbics_entity_focus) | **GET** /factset-rbics/v1/entity-focus | Get RBICS classification for the Focus industry
*EntityFocusApi* | [**get_rbics_entity_focus_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/EntityFocusApi.md#get_rbics_entity_focus_for_list) | **POST** /factset-rbics/v1/entity-focus | Get RBICS classification for the Focus industry
*StructureApi* | [**get_rbics_structure**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/StructureApi.md#get_rbics_structure) | **GET** /factset-rbics/v1/structure | Get the full RBICS Taxonomy Structure Ids, Names, and effective periods.
*StructureApi* | [**get_rbics_structure_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/StructureApi.md#get_rbics_structure_for_list) | **POST** /factset-rbics/v1/structure | Get the full RBICS Taxonomy Structure Ids, Names, and effective periods.


## Documentation For Models

 - [EntityFocus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/EntityFocus.md)
 - [EntityFocusRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/EntityFocusRequest.md)
 - [EntityFocusResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/EntityFocusResponse.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/ErrorResponseSubErrors.md)
 - [Ids](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/Ids.md)
 - [LevelStructure](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/LevelStructure.md)
 - [Levels](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/Levels.md)
 - [RbicsIds](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/RbicsIds.md)
 - [Structure](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/Structure.md)
 - [StructureRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/StructureRequest.md)
 - [StructureResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetRBICS/v1/docs/StructureResponse.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetRBICS.apis and fds.sdk.FactSetRBICS.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetRBICS.api.default_api import DefaultApi`
- `from fds.sdk.FactSetRBICS.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetRBICS
from fds.sdk.FactSetRBICS.apis import *
from fds.sdk.FactSetRBICS.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

